/*
This is a part of the Kidnap LiteStep module source code.

Copyright (C) 2004 Erik Christiansson, aka Sci
erik@alphafish.com
www.alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "main.h"

int initModuleEx(HWND /* hParent */, HINSTANCE /* hInstance */, LPCSTR /* szPath */)
{
	AddBangCommand("!KidnapCapture", bangCapture);
	AddBangCommand("!KidnapRelease", bangRelease);

	return 0;
}

void quitModule(HINSTANCE /* hInstance */)
{
	RemoveBangCommand("!KidnapCapture");
	RemoveBangCommand("!KidnapRelease");

	for(HostageListIter it = lHostages.begin(); it != lHostages.end(); it++) {
		if(IsWindow(it->hWnd)) {
			if(it->bClipChildren = FALSE) {
				SetWindowLong(GetAncestor(it->hWnd, GA_PARENT), GWL_STYLE, GetWindowLong(GetAncestor(it->hWnd, GA_PARENT), GWL_STYLE) ^ WS_CLIPCHILDREN);
			}
			//SetWindowLong(GetAncestor(it->hWnd, GA_PARENT), GWL_STYLE, it->lParentStyle);
			SetParent(it->hWnd, it->hParent);
			SetWindowPos(it->hWnd, NULL, it->rPosition.left, it->rPosition.top, it->rPosition.right - it->rPosition.left, it->rPosition.bottom - it->rPosition.top, SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);
			LSLog(LOG_DEBUG, "Kidnap", "Releasing window...");
		}
		else {
			LSLog(LOG_DEBUG, "Kidnap", "It's not a window...");
		}
	}
}

void bangCapture(HWND hCaller, LPCSTR pszArgs)
{
	HWND hParent = NULL;
	HWND hChild = NULL;
	RECT rPosition;

	int nIndex = 0;
	int nTokens = 0;

	char szFirst[MAX_BANGARGS];
	char szSecond[MAX_BANGARGS];
	char szThird[MAX_BANGARGS];
	char szFourth[MAX_BANGARGS];
	char szFifth[MAX_BANGARGS];
	char szSixth[MAX_BANGARGS];
	char szSeventh[MAX_BANGARGS];
	char szEight[MAX_BANGARGS];

	LPSTR lpszTokens[] = { szFirst, szSecond, szThird, szFourth, szFifth, szSixth, szSeventh, szEight };
	
	nTokens = CommandTokenize(pszArgs, lpszTokens, 8, NULL);

	if(!strncmp(lpszTokens[nIndex], "hwnd:", 5)) {
		hChild = (HWND)atol(lpszTokens[nIndex] + 5);
		if(IsWindow(hChild) == FALSE) {
			hChild = NULL;
		}
		nIndex++;
	}
	else {
		hChild = FindWindowEx2(strlen(lpszTokens[nIndex]) ? lpszTokens[nIndex] : NULL, strlen(lpszTokens[nIndex + 1]) ? lpszTokens[nIndex + 1] : NULL);
		nIndex += 2;
	}
	if(IsWindow(hChild) == FALSE) {
		LSLogPrintf(LOG_DEBUG, "Kidnap", "Could not find child window");
		return;
	}

	if(!strncmp(lpszTokens[nIndex], "hwnd:", 5)) {
		hParent = (HWND)atol(lpszTokens[nIndex] + 5);
		if(IsWindow(hParent) == FALSE) {
			hParent = NULL;
		}
		nIndex++;
	}
	else {
		hParent = FindWindowEx2(strlen(lpszTokens[nIndex]) ? lpszTokens[nIndex] : NULL, strlen(lpszTokens[nIndex + 1]) ? lpszTokens[nIndex + 1] : NULL);
		nIndex += 2;
	}
	if(IsWindow(hParent) == FALSE) {
		LSLogPrintf(LOG_DEBUG, "Kidnap", "Could not find parent window");
		return;
	}

	for(HostageListIter it = lHostages.begin(); it != lHostages.end(); it++) {
		if(it->hWnd == hChild) {
			lHostages.erase(it);
			break;
		}
	}
	KidnapWindow kw;
	kw.hWnd = hChild;
	GetWindowRect(hChild, &kw.rPosition);
	kw.hParent = GetAncestor(hChild, GA_PARENT);
	kw.bClipChildren = (GetWindowLong(hParent, GWL_STYLE) & WS_CLIPCHILDREN != NULL);
	lHostages.push_back(kw);

	POINT pOldPos;
	pOldPos.x = kw.rPosition.left;
	pOldPos.y = kw.rPosition.top;
	ScreenToClient(hParent, &pOldPos);

	if(nTokens >= nIndex + 1) {
		rPosition.left = atoi(lpszTokens[nIndex++]);
	}
	else {
		rPosition.left = pOldPos.x;
	}

	if(nTokens >= nIndex + 1) {
		rPosition.top = atoi(lpszTokens[nIndex++]);
	}
	else {
		rPosition.top = pOldPos.y;
	}

	if(nTokens >= nIndex + 1) {
		rPosition.right = rPosition.left + atoi(lpszTokens[nIndex++]);
	}
	else {
		rPosition.right = rPosition.left + kw.rPosition.right - kw.rPosition.left;
	}

	if(nTokens >= nIndex + 1) {
		rPosition.bottom = rPosition.top + atoi(lpszTokens[nIndex++]);
	}
	else {
		rPosition.bottom = rPosition.top + kw.rPosition.bottom - kw.rPosition.top;
	}

	if(kw.bClipChildren == FALSE) {
		SetWindowLong(hParent, GWL_STYLE, GetWindowLong(hParent, GWL_STYLE) | WS_CLIPCHILDREN);
	}
	SetParent(hChild, hParent);
	SetWindowPos(hChild, NULL, rPosition.left, rPosition.top, rPosition.right - rPosition.left, rPosition.bottom - rPosition.top, SWP_NOZORDER | SWP_NOACTIVATE);
}

void bangRelease(HWND hCaller, LPCSTR pszArgs)
{
	HWND hWnd = NULL;
	int nTokens = 0;

	char szFirst[MAX_BANGARGS];
	char szSecond[MAX_BANGARGS];

	LPSTR lpszTokens[] = { szFirst, szSecond };
	
	nTokens = CommandTokenize(pszArgs, lpszTokens, 2, NULL);

	if(!strncmp(lpszTokens[0], "hwnd:", 5)) {
		hWnd = (HWND)atol(lpszTokens[0] + 5);
		if(IsWindow(hWnd) == FALSE) {
			hWnd = NULL;
		}
	}
	else {
		hWnd = FindWindowEx2(strlen(lpszTokens[0]) ? lpszTokens[0] : NULL, strlen(lpszTokens[1]) ? lpszTokens[1] : NULL);
	}
	if(IsWindow(hWnd) == FALSE) {
		LSLogPrintf(LOG_DEBUG, "Kidnap", "Could not find window");
		return;
	}

	for(HostageListIter it = lHostages.begin(); it != lHostages.end(); it++) {
		if(it->hWnd == hWnd) {
			if(it->bClipChildren = FALSE) {
				SetWindowLong(GetAncestor(hWnd, GA_PARENT), GWL_STYLE, GetWindowLong(GetAncestor(hWnd, GA_PARENT), GWL_STYLE) ^ WS_CLIPCHILDREN);
			}
			//SetWindowLong(GetAncestor(hWnd, GA_PARENT), GWL_STYLE, it->lParentStyle);
			SetParent(hWnd, it->hParent);
			SetWindowPos(hWnd, NULL, it->rPosition.left, it->rPosition.top, it->rPosition.right - it->rPosition.left, it->rPosition.bottom - it->rPosition.top, SWP_NOZORDER | SWP_NOACTIVATE);

			lHostages.erase(it);
			break;
		}
	}
}

HWND FindWindowEx2(LPCSTR pszClass, LPCSTR pszTitle)
{
	HWND hWnd = NULL;
	FindWindowEx2Data fwed;
	fwed.phWnd = &hWnd;
	fwed.pszClass = pszClass;
	fwed.pszTitle = pszTitle;
	EnumWindows(EnumWindowsProc, (LPARAM)&fwed);
	return *(fwed.phWnd);
}

BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	FindWindowEx2Data* pfwed = (FindWindowEx2Data*)lParam;
	char szClass[1024];
	char szTitle[1024];
	if(pfwed->pszClass) {
		GetClassName(hWnd, szClass, 1024);
	}
	if(pfwed->pszTitle) {
		GetWindowText(hWnd, szTitle, 1024);
	}
	if((pfwed->pszClass != NULL ? !stricmp(szClass, pfwed->pszClass) : TRUE) && (pfwed->pszTitle != NULL ? !stricmp(szTitle, pfwed->pszTitle) : TRUE)) {
		*(pfwed->phWnd) = hWnd;
		//LSLogPrintf(LOG_DEBUG, "Kidnap", "'%s' '%s' '%s' '%s'", pfwed->pszClass, szClass, pfwed->pszTitle, szTitle);
		return FALSE;
	}
	EnumChildWindows(hWnd, EnumWindowsProc, lParam);
	return (*pfwed->phWnd == NULL);
}
